@@ -1,11 +1,10 @@
-from .array import X3dDataset, X3dDataArray
+from ._version import get_versions
+from .array import X3dDataArray, X3dDataset
 from .genepsi import gene_epsi_3D
 from .gui import ParametersGui
-from .io import readfield, read_all, write_xdmf
 from .param import param
 from .parameters import Parameters
-
-from ._version import get_versions
+from .sandbox import init_dataset, init_epsi
 
 __version__ = get_versions()["version"]
 del get_versions
